/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Patrick Boenzli
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_OBJECT_MANAGER

#include "object_manager.h"
#include "garbage_collector.h"
#include "list.h"

#include "debug.h"

using namespace std;




/**
 *  constructor, sets everything to zero and define factoryName
 */
FastFactory::FastFactory (ClassID classID, const char* fastFactoryName)
{
  this->setClassID(CL_FAST_FACTORY, "FastFactory");
  this->setName(fastFactoryName);

  this->storedClassID = classID;
  this->next = NULL;

  this->storedDeadObjects = 0;

  FastFactory::registerFastFactory(this);
}

/** a reference to the First FastFactory */
FastFactory* FastFactory::first = NULL;

/**
 *  destructor
 *  clear the Q
 */
FastFactory::~FastFactory ()
{
  if (this == first)
    this->first = NULL;

  if (this->next)
    delete this->next;
}

void FastFactory::registerFastFactory(FastFactory* fastFactory)
{
  PRINTF(4)("Registered FastFactory for '%s'\n", fastFactory->getName());

  if( FastFactory::first == NULL)
    FastFactory::first = fastFactory;
  else
  {
    FastFactory* tmpFac = FastFactory::first;
    while( tmpFac->next != NULL)
      tmpFac = tmpFac->next;
    tmpFac->setNext(fastFactory);
  }
}


/**
 * searches for a FastFactory
 * @param factoryName the Name of the Factory to search for (not used)
 * @param classID the ClassID of the FastFactory to search for
 * @returns true if found, false otherwise.
 */
FastFactory* FastFactory::searchFastFactory(ClassID classID, const char* fastFactoryName)
{
  if (FastFactory::first == NULL)
    return NULL;
   else
   {
     FastFactory* tmpFac = FastFactory::first;
     while (tmpFac != NULL)
     {
       if (tmpFac->storedClassID == classID)
         return tmpFac;
       tmpFac = tmpFac->next;
     }
   }
   return NULL;
}











/**
 *  standard constructor
*/
ObjectManager::ObjectManager ()
{
  this->setClassID(CL_OBJECT_MANAGER, "ObjectManager");
  this->setName("ObjectManager");

}


/**
 *  the singleton reference to this class
*/
ObjectManager* ObjectManager::singletonRef = NULL;

/**
 *  standard deconstructor
*/
ObjectManager::~ObjectManager ()
{
  ObjectManager::singletonRef = NULL;
}

/**
 *  outputs some simple debug information about the ObjectManage
*/
void ObjectManager::debug() const
{
  PRINT(0)("\n==========================| ObjectManager::debug() |===\n");
/* PRINT(0)("=  Number of registerable classes: %i\n", CL_NUMBER );
 PRINT(0)("=  Currently cached objects: \n");
 for(int i = 0; i < CL_NUMBER; ++i)
   {
      if( this->managedObjectList[i] != NULL)
        PRINT(0)("=   o Class Nr. %i has cached %i object(s)\n", i, this->managedObjectList[i]->getSize());
      else
        PRINT(0)("=   o Class Nr. %i has cached 0 object(s)\n", i);
    }*/
  PRINT(0)("=======================================================\n");
}


